import { ExecuteOperation } from 'executable-openapi-types';
import { OpenAPIObject } from 'openapi3-ts';
import { HandlersMap } from './types';
export interface CreateRouterOptions {
    /**
     * A JSON reference resolver that'll be used to resolve
     * JSON references found in `document`.
     *
     * If not provided, `createRouter` will use a simple resolver
     * that only resolves local JSON pointer.
     */
    documentRefResolver?: RefResolver;
}
declare type RefResolver = ($ref: string) => Promise<unknown>;
/**
 * Create a router, ie an [execution operation function](executable-openapi-types/modules#executeoperation)
 * that'll route operation to operation handlers given in `handlers`.
 *
 * @param document - An OpenAPI document describing the service
 * @param handlers - A map of handlers
 * @param options -  `CreateRouterOptions`
 */
export declare function createRouter<TContext = undefined>(document: OpenAPIObject, handlers: HandlersMap<TContext>, options?: CreateRouterOptions): ExecuteOperation<TContext>;
export {};
