"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRouter = void 0;
const bath_1 = require("bath");
const openapi_document_local_ref_resolver_1 = require("openapi-document-local-ref-resolver");
const utils_1 = require("./utils");
/**
 * Create a router, ie an [execution operation function](executable-openapi-types/modules#executeoperation)
 * that'll route operation to operation handlers given in `handlers`.
 *
 * @param document - An OpenAPI document describing the service
 * @param handlers - A map of handlers
 * @param options -  `CreateRouterOptions`
 */
function createRouter(document, handlers, options = {}) {
    const paths = Object.entries(document.paths)
        .map(([path, pathItemObject]) => [path, bath_1.params(path), pathItemObject]);
    const defaultHandler = handlers.default !== undefined ? handlers.default : notImplementedHandler;
    const resolveRef = options.documentRefResolver !== undefined ? options.documentRefResolver : openapi_document_local_ref_resolver_1.localRefResolver(document);
    const execute = (executionRequest, context) => __awaiter(this, void 0, void 0, function* () {
        var _a, _d, _e, _f;
        const matchedPaths = paths
            .map(([path, match, pathItemObject]) => [path, match(executionRequest.path), pathItemObject])
            .filter(([, pathParameters]) => {
            if (pathParameters !== null)
                return true;
            return false;
        })
            // take the most contrete path
            .sort(([path1], [path2]) => utils_1.pathConcretenessCompareFunction(path1, path2));
        // not a match
        if (matchedPaths.length === 0)
            return null;
        const [path, pathParameters, pathItemObject] = matchedPaths[0];
        if (pathParameters === null)
            return null;
        if (pathItemObject === undefined)
            return null;
        let resolvedPathItemObject;
        if (pathItemObject.$ref !== undefined) {
            resolvedPathItemObject = (yield resolveRef(pathItemObject.$ref));
        }
        else {
            resolvedPathItemObject = pathItemObject;
        }
        const operationObject = resolvedPathItemObject[executionRequest.method];
        if (operationObject === undefined)
            return null;
        const info = {
            document,
            executionRequest,
            operationObject,
            path,
            pathItemObject: resolvedPathItemObject
        };
        const parameters = {
            path: pathParameters,
            query: executionRequest.query
        };
        // find a handler
        const pathHandler = (_a = handlers === null || handlers === void 0 ? void 0 : handlers.paths) === null || _a === void 0 ? void 0 : _a[path][executionRequest.method];
        const operationHandler = operationObject.operationId !== undefined ? (_d = handlers === null || handlers === void 0 ? void 0 : handlers.operations) === null || _d === void 0 ? void 0 : _d[operationObject.operationId] : undefined;
        const handler = (pathHandler !== undefined) ? pathHandler : operationHandler;
        if (handler === undefined) {
            return yield defaultHandler(parameters, (_e = executionRequest.body) === null || _e === void 0 ? void 0 : _e.content, context, info);
        }
        return yield handler(parameters, (_f = executionRequest.body) === null || _f === void 0 ? void 0 : _f.content, context, info);
    });
    return execute;
}
exports.createRouter = createRouter;
const notImplementedHandler = (_p, _b, _c, { path }) => {
    throw new Error(`No handler found for ${path}`);
};
//# sourceMappingURL=createRouter.js.map